#pragma once

#include <fstream>

using namespace std;

struct DataPoint {
	int pixel_dist;
	double actual_dist;
};

const double LESS_THAN_MIN_DIST = -1.0;
const double GREATER_THAN_MAX_DIST = -2.0;

class DistanceLookUp
{
public:

	DistanceLookUp(void)
	{
		data.clear();
	}

	~DistanceLookUp(void)
	{
		data.clear();
	}

	DistanceLookUp(string filename)
	{
		load(filename);
	}

	void insest_point(int pixel_dist, double actual_dist)
	{
		int size = (int)data.size();
		DataPoint insert, temp;
		int position = find(pixel_dist);

		if (position == -1) {
			insert.pixel_dist = pixel_dist;
			insert.actual_dist = actual_dist;

			for (int i = 0; i < size; i++) {
				if (insert.pixel_dist < data[i].pixel_dist) {
					temp = data[i];
					data[i] = insert;
					insert = temp;
				}
			}

			data.push_back(insert);
		}
		else {
			data[position].actual_dist += actual_dist;
			data[position].actual_dist /= 2;
		}
	}

	double get_distance(int pixel_dist)
	{
		int size = (int)data.size();
		int position = 0;
		double distance = LESS_THAN_MIN_DIST;
		
		for (position = 0; position < size; position++) {
			if (!(data[position].pixel_dist < pixel_dist)) {
				break;
			}
		}

		if (position == 0) {
			if (data[position].pixel_dist == pixel_dist) {
				distance = data[position].actual_dist;
			}
		}
		else if (position == size) {
			distance = GREATER_THAN_MAX_DIST;
		}
		else {
			if (data[position].pixel_dist == pixel_dist) {
				distance = data[position].actual_dist;
			}
			else {
				distance = (data[position - 1].actual_dist 
					       + data[position].actual_dist) / 2;
			}
		}

		return distance;
	}

	void load(string filename)
	{
		data.clear();

		ifstream in(filename.c_str());
		DataPoint insert;

		if (!in.is_open()) {
			return;
		}

		while (in) {
			in >> insert.pixel_dist;
			in >> insert.actual_dist;

			data.push_back(insert);
		}

		in.close();
	}

	void save(string filename)
	{
		ofstream out(filename.c_str());
		if (!out.is_open()) {
			return;
		}
		int size = (int)data.size();

		for (int i = 0; i < size; i++) {
			out << data[i].pixel_dist << " ";
			out << data[i].actual_dist << "\n";
		}

		out.close();
	}

private:

	int find(int pixel_dist)
	{
		int size = (int)data.size();

		for (int i = 0; i < size; i++) {
			if (data[i].pixel_dist == pixel_dist) {
				return i;
			}
		}

		return -1;
	}

	vector<DataPoint> data;
};
