#pragma once

#include "RobotImage.h"
#include <vector>

using namespace std;

struct Location {
	int x;
	int y;
};

struct Blob {
	int x;
	int y;
	int mass;
};

const BYTE ON = 255;
const BYTE OFF = 0;
const int EXPAND_AMOUNT = 5;

vector<Blob> searchImage(RobotImage& image);
RobotImage* thresholdImage(RobotImage& image);
Blob mark_blob(RobotImage& image, int x_pixel, int y_pixel);
void expandWhite(RobotImage& image);
inline void expandWhitePixel(RobotImage& image, int x_pixel,
							 int y_pixel, int amount);
void expandBlack(RobotImage& image);
inline void expandBlackPixel(RobotImage& image, int x_pixel,
							 int y_pixel, int amount);
inline bool thresholdPixel(BYTE hue, BYTE sat, BYTE val);


vector<Blob> searchImage(RobotImage& image)
{
	int height = image.get_height();
	int width = image.get_width();

	vector<Blob> results;
	Blob found;

	RobotImage* thresholded = thresholdImage(image);
	thresholded->save("output\\thresholded hsv image.bmp");

	expandWhite(*thresholded);
	thresholded->save("output\\thresholded hsv image white expanded.bmp");
	//expandBlack(*thresholded);

	//thresholded->save("output\\thresholded hsv image processed.bmp");


	for (int y = 0; y < height; y++) {
		for (int x = 0; x < width; x++) {
			if (thresholded->isWhite(x,y)) {
				found = mark_blob(*thresholded, x, y);
				results.push_back(found);
			}
		}
	}

	delete thresholded;

	return results;
}

RobotImage* thresholdImage(RobotImage& image)
{
	int height = image.get_height();
	int width = image.get_width();
	RobotImage* result = new RobotImage(width, height);
	BYTE hue = 0;
	BYTE sat = 0;
	BYTE val = 0;

	for (int y = 0; y < height; y++) {
		for (int x = 0; x < width; x++) {
			image.get_pixelHSV(x, y, hue, sat, val);

			if (thresholdPixel(hue, sat, val)) {
				result->set_pixelRGB(x, y, ON, ON, ON);
			}
			else {
				result->set_pixelRGB(x, y, OFF, OFF, OFF);
			}
		}
	}

	return result;
}

Blob mark_blob (RobotImage& image, int x_pixel, int y_pixel)
{
	cout << "Finding blob!" << endl;
	int height = image.get_height();
	int width = image.get_width();

	Blob result;
	vector<Location> stack;
	vector<Location> blob_pixels;
	Location current;
	int x, y;

	result.x = 0;
	result.y = 0;
	result.mass = 0;

	if (!image.isWhite(x_pixel, y_pixel)) {
		return result;
	}

	current.x = x_pixel;
	current.y = y_pixel;

	image.set_pixelRGB(x_pixel, y_pixel, OFF, OFF, OFF);
	stack.push_back(current);

	while (!stack.empty()) {

		current = stack.back();
		stack.pop_back();

		x = current.x;
		y = current.y;

		blob_pixels.push_back(current);

		current.x--;
		x--;
		if ((0 <= x && x < width) && (0 <= y && y < height) && image.isWhite(x, y)) {
			cout << "(" << x << ", " << y << ")" << endl;
			if (!(0 <= x && x < width) || !(0 <= y && y < height)) {
				continue;
			}
			stack.push_back(current);
			image.set_pixelRGB(x, y, OFF, OFF, OFF);
		}

		current.x += 2;
		x += 2;
		if ((0 <= x && x < width) && (0 <= y && y < height) && image.isWhite(x, y)) {
			cout << "(" << x << ", " << y << ")" << endl;
			stack.push_back(current);
			image.set_pixelRGB(x, y, OFF, OFF, OFF);
		}
		current.x--;
		x--;

		current.y--;
		y--;
		if ((0 <= x && x < width) && (0 <= y && y < height) && image.isWhite(x, y)) {
			cout << "(" << x << ", " << y << ")" << endl;
			if (!(0 <= x && x < width) || !(0 <= y && y < height)) {
				continue;
			}
			stack.push_back(current);
			image.set_pixelRGB(x, y, OFF, OFF, OFF);
		}

		current.y += 2;
		y += 2;
		if ((0 <= x && x < width) && (0 <= y && y < height) && image.isWhite(x, y)) {
			cout << "(" << x << ", " << y << ")" << endl;
			if (!(0 <= x && x < width) || !(0 <= y && y < height)) {
				continue;
			}
			stack.push_back(current);
			image.set_pixelRGB(x, y, OFF, OFF, OFF);
		}
	}

	while (!blob_pixels.empty()) {
		current = blob_pixels.back();
		blob_pixels.pop_back();
		result.mass++;
		result.x += current.x;
		result.y += current.y;
	}

	result.x /= result.mass;
	result.y /= result.mass;

	cout << "Blob found!" << endl;
	return result;
}

void expandWhite (RobotImage& image)
{
	int height = image.get_height();
	int width = image.get_width();
	vector<Location> pixels;
	Location temp;

	for (int y = 0; y < height; y++) {
		for (int x = 0; x < width; x++) {
			if (image.isWhite(x, y)) {
				temp.x = x;
				temp.y = y;
				pixels.push_back(temp);
			}
		}
	}

	int numpixels = (int)pixels.size();
	for (int i = 0; i < numpixels; i++) {
		temp = pixels[i];
		expandWhitePixel(image, temp.x, temp.y, EXPAND_AMOUNT);
	}
}

inline void expandWhitePixel(RobotImage& image, int x_pixel,
							 int y_pixel, int amount)
{
	int height = image.get_height();
	int width = image.get_width();

	int x_min = max(x_pixel - amount, 0);
	int x_max = min(x_pixel + amount, width - 1);
	int y_min = max(y_pixel - amount, 0);
	int y_max = min(y_pixel + amount, height - 1);

	for (int y = y_min; y <= y_max; y++) {
		for (int x = x_min; x <= x_max; x++) {
			image.set_pixelRGB(x, y, ON, ON, ON);
		}
	}
}

void expandBlack(RobotImage& image)
{
	int height = image.get_height();
	int width = image.get_width();
	vector<Location> pixels;
	Location temp;

	for (int y = 0; y < height; y++) {
		for (int x = 0; x < width; x++) {
			if (!image.isWhite(x, y)) {
				temp.x = x;
				temp.y = y;
				pixels.push_back(temp);
			}
		}
	}

	int numpixels = (int)pixels.size();
	for (int i = 0; i < numpixels; i++) {
		temp = pixels[i];
		expandBlackPixel(image, temp.x, temp.y, EXPAND_AMOUNT);
	}
}

inline void expandBlackPixel(RobotImage& image, int x_pixel,
							 int y_pixel, int amount)
{
	int height = image.get_height();
	int width = image.get_width();

	int x_min = max(x_pixel - amount, 0);
	int x_max = min(x_pixel + amount, width - 1);
	int y_min = max(y_pixel - amount, 0);
	int y_max = min(y_pixel + amount, height - 1);

	for (int y = y_min; y <= y_max; y++) {
		for (int x = x_min; x <= x_max; x++) {
			image.set_pixelRGB(x, y, OFF, OFF, OFF);
		}
	}
}

inline bool thresholdPixel(BYTE hue, BYTE sat, BYTE val)
{
	return (   (0 < sat && sat < 80)
			&& (200 < val)
			&& ((hue < 20 || 235 < hue) && hue != UNDEFINED_HUE));
}