#pragma once

#include "stdafx.h"
#include <string>
#include <iostream>
#include <math.h>
#include <float.h>
#include <assert.h>
#include <atlimage.h>
#include "resource.h"

using namespace std;

typedef unsigned char BYTE;

const int NUM_BPP = 24;
const int BYTES_PP = 3;
const int MAX_BYTE = 255;
const BYTE UNDEFINED_HUE = 255;
const BYTE MAX_HUE = 254;
const double M_PI = 3.14159265358979323846;

enum RGB_COLOR_BYTE { BLUE = 0, GREEN = 1, RED = 2 };
enum HSV_COLOR_BYTE { HUE = 0, SAT = 1, VAL = 2 };

class RobotImage
{
public:

	RobotImage()
	{
		RGB = true;
		picture = new CImage;
		whiteThreshold = 127;
	}

	RobotImage(int _width, int _height)
	{
		RGB = true;
		picture = new CImage;
		picture->Create(_width, _height, NUM_BPP);
		whiteThreshold = 127;
	}

	RobotImage(RobotImage& _assign)
	{
		*this = _assign;
	}

	~RobotImage (void)
	{
		if (picture != NULL) {
			delete picture;
		}
	}

	RobotImage& operator = (RobotImage& right_arg)
	{
		delete picture;

		RGB = right_arg.RGB;
		whiteThreshold = right_arg.whiteThreshold;
		
		int height = right_arg.get_height();
		int width = right_arg.get_width();
		BYTE red, green, blue;

		picture = new CImage;
		picture->Create(width, height, NUM_BPP);

		for (int y = 0; y < height; y++) {
			for (int x = 0; x < width; x++) {
				right_arg.get_pixelRGB(x, y, red, green, blue);
				set_pixelRGB(x, y, red, green, blue);
			}
		}
	
		return *this;
	}

	bool load (string filename)
	{
		RGB = true;
		picture->Load(filename.c_str());
		setBPP();

		return true;
	}

	bool save (string filename)
	{
		//if (!RGB) {
		// 	convert_to_RGB();
		//}
		
		picture->Save(filename.c_str());
		return true;
	}
	
	inline void set_RGB(bool _RGB)
	{
		RGB = _RGB;
	}

	inline bool isWhite(int x, int y)
	{
		BYTE* pixel = (BYTE*)picture->GetPixelAddress(x,y);
		BYTE intensity;
		if (RGB) {
			intensity = (pixel[RED] + pixel[GREEN] + pixel[BLUE]) / 3;
		}
		else {
			intensity = pixel[VAL];
		}
		return (intensity > whiteThreshold);
	}

	inline bool get_pixelRGB(int x, int y, BYTE& red, BYTE& green, BYTE& blue)
	{
		if (RGB) {
			BYTE* pixel = (BYTE*)picture->GetPixelAddress(x,y);
			
			red = pixel[RED];
			green = pixel[GREEN];
			blue = pixel[BLUE];

			return true;
		}
		else {
			return false;
		}
	}

	inline bool set_pixelRGB(int x, int y, BYTE red, BYTE green, BYTE blue)
	{
		if (RGB) {
			BYTE* pixel = (BYTE*)picture->GetPixelAddress(x,y);
	
			pixel[RED] = red;
			pixel[GREEN] = green;
			pixel[BLUE] = blue;

			return true;
		}
		else {
			return false;
		}
	}

	inline bool get_pixelHSV(int x, int y, BYTE& hue, BYTE& sat, BYTE& val)
	{
		if (!RGB) {
			BYTE* pixel = (BYTE*)picture->GetPixelAddress(x,y);
			
			hue = pixel[HUE];
			sat = pixel[SAT];
			val = pixel[VAL];
			
			return true;
		}
		else {
			return false;
		}
	}

	inline bool set_pixelHSV(int x, int y, BYTE hue, BYTE sat, BYTE val)
	{
		if (!RGB) {
			BYTE* pixel = (BYTE*)picture->GetPixelAddress(x,y);
	
			pixel[HUE] = hue;
			pixel[SAT] = sat;
			pixel[VAL] = val;

			return true;
		}
		else {
			return false;
		}
	}

	inline void convert_to_RGB()
	{
		if (!RGB) {
			int width = picture->GetWidth();
			int height = picture->GetHeight();
			for (int y = 0; y < height; y++) {
				for (int x = 0; x < width; x++) {
					HSV_to_RGB((BYTE*)picture->GetPixelAddress(x, y));
				}
			}
			RGB = true;
		}
	}

	inline void convert_to_HSV()
	{
		if (RGB) {
			int width = picture->GetWidth();
			int height = picture->GetHeight();
			for (int y = 0; y < height; y++) {
				for (int x = 0; x < width; x++) {
					RGB_to_HSV((BYTE*)picture->GetPixelAddress(x, y));
				}
			}
			RGB = false;
		}
	}

	inline int get_height(void)
	{
		return picture->GetHeight();
	}

	inline int get_width(void)
	{
		return picture->GetWidth();
	}

private:

	void setBPP()
	{
		if (picture->GetBPP() == NUM_BPP) {
			return;
		}

		int width = picture->GetWidth();
		int height = picture->GetHeight();

		CImage* temp = new CImage;
		temp->Create(width, height, NUM_BPP);

		for (int y = 0; y < height; y++) {
			for (int x = 0; x < width; x++) {
				temp->SetPixel(x, y, picture->GetPixel(x, y));
			}
		}

		picture->Destroy();
		delete picture;

		picture = temp;
	}

	inline void RGB_to_HSV(BYTE* pixelAddress)
	{
		BYTE red = pixelAddress[RED];
		BYTE green = pixelAddress[GREEN];
		BYTE blue = pixelAddress[BLUE];

		int max = 0;
		int min = 0;

		double hue = 0.0;
		double sat = 0.0;
		int val = 0;

		max = max(red, green);
		min = min(red, green);

		if (blue > max) {
			max = blue;
		}
		else if (blue < min) {
			min = blue;
		}

		val = (red + green + blue) / 3;

		if (max != min) {
			sat = 1.0 - ((double)min / val);

			hue = compute_hue(red, green, blue);

			sat *= MAX_BYTE;
			hue *= MAX_HUE / (2 * M_PI);
			hue = floor(hue);
		}
		else {
			sat = 0.0;
			hue = (double) UNDEFINED_HUE;
		}

		pixelAddress[HUE] = (BYTE) hue;
		pixelAddress[SAT] = (BYTE) sat;
		pixelAddress[VAL] = (BYTE) val;
	}

	inline void HSV_to_RGB(BYTE* pixelAddress) 
	{
		BYTE hue = pixelAddress[HUE];
		BYTE sat = pixelAddress[SAT];
		BYTE val = pixelAddress[VAL];

		if (hue == UNDEFINED_HUE) {
			pixelAddress[RED] = val;
			pixelAddress[GREEN] = val;
			pixelAddress[BLUE] = val;
			return;
		}

		double red;
		double green;
		double blue;

		double d_hue = 2 * M_PI * ((double)++hue / MAX_BYTE) ;
		double d_sat = (double)sat / MAX_BYTE;
		double d_val = (double)val / MAX_BYTE;

		if (d_hue < (2.0 / 3.0) * M_PI) {
			red = d_val * (1.0 + (d_sat * cos(d_hue)) / cos((M_PI / 3.0) - d_hue));
			blue = d_val * (1.0 - d_sat);
			green = (3.0 * d_val) - (red + blue);;
		}
		else if (d_hue < (4.0 / 3.0) * M_PI) {
			d_hue -= (2.0 / 3.0) * M_PI;
			red = d_val * (1.0 - d_sat);
			green = d_val * (1.0 + (d_sat * cos(d_hue)) / cos((M_PI / 3.0) - d_hue));
			blue = (3.0 * d_val) - (red + green);;
		}
		else {
			d_hue -= (4.0 / 3.0) * M_PI;
			green = d_val * (1.0 - d_sat);
			blue = d_val * (1.0 + (d_sat * cos(d_hue)) / cos((M_PI / 3.0) - d_hue));
			red = (3.0 * d_val) - (green + blue);
		}

		pixelAddress[RED] = (BYTE) (red * MAX_BYTE);
		pixelAddress[GREEN] = (BYTE) (green * MAX_BYTE);
		pixelAddress[BLUE] = (BYTE) (blue * MAX_BYTE);
	}

	inline double compute_hue (const BYTE& red, const BYTE& green, const BYTE& blue)
	{
		double r_g = (red - green) / (double)MAX_BYTE;
		double r_b = (red - blue) / (double)MAX_BYTE;
		double g_b = (green - blue) / (double)MAX_BYTE;

		double numerator = (0.5) * (r_g + r_b);
		double denominator = sqrt((r_g * r_g) + (r_b * g_b));

		double hue = acos(numerator / denominator);

		if (blue > green) {
			hue = 2 * M_PI - hue;
		}

		return hue;
	}

	CImage* picture;
	bool RGB;
	BYTE whiteThreshold;
};
