// vision.cpp : Defines the entry point for the console application.
//

#include "stdafx.h"
#include "RobotImage.h"
#include "ImageSearch.h"
#include "DistanceLookUp.h"
#include <fstream>
#include "py_funcs.h"

//const string RIGHT_IN_FILE = "sunset.jpg";
const string IN_FILE = "r:\\right.BMP";
//const string LEFT_IN_FILE = "r:\\left.BMP";
const string HSV_OUT = "output\\hsv_output_test.bmp";
const string TEST_OUT = "output\\output_test.bmp";
const string HUE_OUT = "output\\hue_output.bmp";
const string SAT_OUT = "output\\sat_output.bmp";
const string VAL_OUT = "output\\val_output.bmp";
const string THRESHOLD_OUT = "output\\thresholded hsv image.bmp";
const string GAUGE_FILE = "output\\distance gauge info.txt";
const int MARKER_RADIUS = 6;
const int MARGIN = 0;

vector<string> get_filenames(void);
void write_hsv(RobotImage& hsv);
void mark_blobs(RobotImage& image, vector<Blob>& blobs);
string add_to_filename(string original, string toAdd);

int _tmain(int argc, _TCHAR* argv[])
{
	DistanceLookUp gauge(GAUGE_FILE);
	int width = 0, height = 0;
	RobotImage input, hsv;
	string quit = "";
	vector<Blob> blobs;
	double distance = 200.0;

	//cout << "Beginning calibration." << endl;
	//cout << "To stop calibrating, type 'done'." << endl;
	//while (distance > 0.0) {

	//	cout << "Please take picture and specify distance." << endl;
	//	cin >> distance;

	//	input.load(IN_FILE);
	//	hsv = input;
	//	hsv.convert_to_HSV();

	//	blobs = searchImage(hsv);
	//	if ((int)blobs.size() == 1) {
	//		gauge.insest_point(blobs[0].x, distance);
	//	}
	//}


	cout << "Beginning Testing." << endl;
	cout << "To stop testing, type 'done'." << endl;
	while (cin >> quit) {
		if (quit == "done") {
			break;
		}

		input.load(IN_FILE);
		hsv = input;
		hsv.convert_to_HSV();

		blobs = searchImage(hsv);
		if ((int)blobs.size() == 1) {
			distance = gauge.get_distance(blobs[0].x);

			cout << "Distance determined to be " << distance << endl;
		}
		else {
			cout << "Can't determine distance." << endl;
		}

		cout << "To stop testing, type 'done'." << endl;
	}

	//gauge.save(GAUGE_FILE);
	return 0;
}

vector<string> get_filenames (void)
{
	vector<string> filenames;
	filenames.push_back("R:\\30.bmp");
	filenames.push_back("R:\\50.bmp");
	filenames.push_back("R:\\70.bmp");
	filenames.push_back("R:\\90.bmp");
	int numfiles = (int) filenames.size();

	cout << "Going to read in:" << endl;
	for (int i = 0; i < numfiles; i++) {
		cout << filenames[i] << endl;
	}
	return filenames;
}

void write_hsv (RobotImage& hsv)
{
	int width = hsv.get_width();
	int height = hsv.get_height();
	BYTE hue, sat, val;

	RobotImage ihue(width, height);
	RobotImage isat(width, height);
	RobotImage ival(width, height);

	for (int y = 0; y < height; y++) {
		for (int x = 0; x < width; x++) {
			hsv.get_pixelHSV(x, y, hue, sat, val);
			ihue.set_pixelRGB(x, y, hue, hue, hue);
			isat.set_pixelRGB(x, y, sat, sat, sat);
			ival.set_pixelRGB(x, y, val, val, val);
		}
	}
	
	hsv.save(HSV_OUT);
	ihue.save(HUE_OUT);
	isat.save(SAT_OUT);
	ival.save(VAL_OUT);
}

void mark_blobs (RobotImage& image, vector<Blob>& blobs)
{
	int height = image.get_height();
	int width = image.get_width();
	int x, y;

	Blob laser_loc;
	while (!blobs.empty()) {
		
		laser_loc = blobs.back();
		blobs.pop_back();

		x = laser_loc.x;
		y = laser_loc.y;

		if ((x < MARGIN || width - MARGIN < x) ||
			(y < MARGIN || width - MARGIN < y)) {
			continue;
		}

		int x_max = min(x + MARKER_RADIUS, width);
		int x_min = max(x - MARKER_RADIUS, 0);
		int y_max = min(y + MARKER_RADIUS, height);
		int y_min = max(y - MARKER_RADIUS, 0);	

		for (int y = y_min; y < y_max; y++) {
			for (int x = x_min; x < x_max; x++) {
				image.set_pixelRGB(x, y, 255, 0, 0);
			}
		}
	}
}

string add_to_filename (string original, string toAdd)
{
	string result = "";
	char* temp = (char*)original.c_str();
	int index = 0;

	while (temp[index] != '\0') {
		if (temp[index] == '.') {
			result += toAdd;
			toAdd = "";
		}
		result += temp[index++];
	}

	return result;
}